#!/usr/bin/env python
#
# -------------------------------------------------------------------------------------
#
# Copyright (c) 2024 Skinnyhouse
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
#
#    - Redistributions of source code must retain the above copyright notice, this 
#    list of conditions and the following disclaimer.
#    - Redistributions in binary form must reproduce the above copyright notice, 
#    this list of conditions and the following disclaimer in the documentation and/or 
#    other materials provided with the distribution.
#    - Neither the name of the author nor the names of its contributors may be used 
#    to endorse or promote products derived from this software without specific prior 
#    written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
# SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
# DAMAGE.
#
# -------------------------------------------------------------------------------------
#
# This file is a basic Python plug-in for GIMP.
# It creates highlights from selections
# It can be executed by selecting the menu option: 'Filters>Custom Fonts>Decorating Tools>Custom Font Selection Highlight v 0.1...
# Version 0.1 options for:
#    Highlight size
#    Counting selection size available in increments
#    Alternative size for nominated characters
#    Adding colour
#    Applying modes
#    Opacity
#    Setting highlight offsets
    


from gimpfu import *
#from array import array
#import math
#import random

#from gtk import gdk
#w = gdk.screen_width()
#h = gdk.screen_height()

# Some global colours we may need to define


# Some global colours we may need to define

black = gimpcolor.RGB(0, 0, 0)
white = gimpcolor.RGB(255, 255, 255)
darkestgrey = gimpcolor.RGB(20, 20, 20)
verydarkgrey = gimpcolor.RGB(50, 50, 50)
darkgrey = gimpcolor.RGB(75, 75, 75)
midgrey2 = gimpcolor.RGB(105, 105, 105)
midgrey = gimpcolor.RGB(125, 125, 125)
lightgrey = gimpcolor.RGB(170, 170, 170)
verylightgrey = gimpcolor.RGB(222, 222, 222)
red = gimpcolor.RGB(220,10,10)	



def do_stuff(image, layer, shrinkage, alt_size_char_list, alt_size, step_amount, blur_amount, colour_option, new_mode, opacity, x_offset, y_offset):
    #### Main Filter Proc
	
    # Store the GIMP's settings so they can be restored
    gimp.context_push()
    # Set start undo point
    pdb.gimp_image_undo_group_start(image)	
	
    #####
    ##### TEMP VALUES
	
	
		
    dummy = 0
    active_layer_id = pdb.gimp_image_get_active_layer(image)
	
    mode = fn_get_mode(new_mode)
	
    loop_ptr = 1
    first_char_layer_name = "none"
    num_layers, layer_ids = pdb.gimp_image_get_layers(image)
    loop = num_layers - loop_ptr
    layer_names_list = image.layers
    original_active_layer = image.active_layer
		
    num_multi_layers,other_layers,num_char_layers = 0,0,0
    visi_loop, first_char_layer  = 0, -1

    while visi_loop < num_layers:
	    # get info about layer we are working on
        layer_name = pdb.gimp_item_get_name(layer_names_list[visi_loop])
        layer_name = layer_name.decode('utf-8')
        # check to see if the current layer is a custom font character layer
        test_for_custom_layer = fn_test_for_custom_layer(image, layer_names_list[visi_loop], layer_name)
			
        if test_for_custom_layer:
            num_char_layers += 1
            if first_char_layer < 0: 
                first_char_layer = visi_loop
                first_char_layer_name = layer_name
                layer_names_list[visi_loop].set_offsets(0, 0)
				
            
        pdb.gimp_item_set_visible(layer_names_list[visi_loop],False) # turn off visibility
        visi_loop += 1
		
		
    while loop >= 0:
	    # get info about layer we are working on for custom font mode
        layer_id = layer_ids[loop]
        layer_name = layer_names_list[loop]
	
        # cater for unicode characters above normal range
        display_layer_name =  pdb.gimp_item_get_name(layer_names_list[loop])
        display_layer_name = display_layer_name.decode('utf-8')	
	
        # check we have the layer id of the currently active layer
        pdb.gimp_image_set_active_layer(image, layer_name)
        layer_id = pdb.gimp_image_get_active_layer(image)

        # check to see if the current layer is a custom font character layer
        test_for_custom_layer = fn_test_for_custom_layer(image, layer_id, display_layer_name)
	
	
        if test_for_custom_layer:
            # ensure we have the active layer and make it visible
            active_layer_id = pdb.gimp_image_get_active_layer(image)
            pdb.gimp_item_set_visible(active_layer_id, True)
            
            ##### place Custom Font Code here
            shrink_ctr = 0
            pdb.gimp_image_select_item(image, 2, layer_id)# alpha select character
            non_empty, x1, y1, x2, y2 = pdb.gimp_selection_bounds(image)
            while non_empty: # loop to find out the maximum number of times each character can shrink for shrink_ctr value
                pdb.gimp_selection_shrink(image, step_amount)
                shrink_ctr+=step_amount
                non_empty, x1, y1, x2, y2 = pdb.gimp_selection_bounds(image)
                #if shrink_ctr>999: non_empty=False
            if shrink_ctr>0: # selection available
                if display_layer_name[0] in alt_size_char_list: # check for alternative character size required
                    shrink_amount = (float(shrink_ctr)/100.0)*(100.0-float(alt_size))
                    #pdb.gimp_message("alt shrink_amount for " + display_layer_name[0] + " = " + str(shrink_amount) + " : shrink_ctr = " + str(shrink_ctr ))
                else:
                    shrink_amount = (float(shrink_ctr)/100.0)*(100.0-float(shrinkage)) # calculate shrink amount
                    #pdb.gimp_message("shrink_amount for " + display_layer_name[0] + " = " + str(shrink_amount) + " : shrink_ctr = " + str(shrink_ctr ))
                
                copy_layer = pdb.gimp_layer_copy(layer_id, True) # copy and add and clear for a new layer
                pdb.gimp_image_insert_layer(image, copy_layer, None, -1)
                pdb.gimp_drawable_edit_clear(copy_layer)
                pdb.gimp_image_select_item(image, 2, layer_id) # select the contents of the original character layer
                pdb.gimp_selection_shrink(image, shrink_amount) # shrink the selection by user value
                pdb.gimp_drawable_edit_fill(copy_layer, colour_option) # fill selection with selected colour
                pdb.gimp_layer_set_mode(copy_layer, mode) # change layer mode according to user
                pdb.gimp_layer_set_opacity(copy_layer, opacity)# change layer opacity according to user
                pdb.gimp_selection_none(image)
                pdb.gimp_image_select_item(image, 2, layer_id) # add a selection to constrain blur
                if blur_amount>0: pdb.plug_in_gauss(image, copy_layer, blur_amount*5, blur_amount*5, 0) # blur if required
                pdb.gimp_selection_none(image)
                if x_offset<>0 or y_offset<> 0: # move highlight layer if necessary
                    pdb.gimp_drawable_offset(copy_layer, False, 1, x_offset, y_offset)
                    pdb.plug_in_autocrop_layer(image, copy_layer) # crop the highlight layer
                layer_id = pdb.gimp_image_merge_down(image, copy_layer, 2) # and merge the highlight with the original character
            
                
            pdb.gimp_selection_none(image)
            pdb.gimp_item_set_visible(layer_id, False) # turn off layer visibilty 
        loop_ptr += 1
        loop = num_layers - loop_ptr
		
		
    if first_char_layer_name <> "none": # the character layer to make visible on completion
        first_char_layer_id = pdb.gimp_image_get_layer_by_name(image, first_char_layer_name)
        pdb.gimp_item_set_visible(first_char_layer_id, True)
        pdb.gimp_image_set_active_layer(image,first_char_layer_id)
	
    # Set end undo point
    pdb.gimp_image_undo_group_end(image)

    # Restore the GIMP's settings
    gimp.context_pop()
	
######################################################################################################################################

    #### Useful Functions
# -------------------------------------------------------------------------------------
# -------------------------------------------------------------------------------------

def fn_test_for_custom_layer(img,l_id,l_name):
    # checks that the current layer is compatible with custom fonts
    test = False
    if len(l_name) == 1:
        test = True
    elif len(l_name) == 5 or len(l_name) == 6:
        
        if ".png" in l_name:
            test = True
            if len(l_name) == 6:
                part_name_val = unicode(l_name[0:2])
                if part_name_val <= 65535: test = False
    # select the contents of the layer
    pdb.gimp_image_select_item(img,2,l_id)
    # Test to see if the selection actually contains something
    sel_is_empty = pdb.gimp_selection_is_empty(img)
    # If there are no pixels to work with flag the layer as not viable to work on
    if sel_is_empty: test = False
    # Deselect
    pdb.gimp_selection_none(img)
    return test
	
######################################################################################################################################
	
def fn_selection_info(im):
    # returns selection info
    selection,x1,y1,x2,y2 = pdb.gimp_selection_bounds(im)
    s_width, s_height = x2 - x1, y2 - y1
	
    return selection, x1, x2, y1, y2, s_width, s_height


######################################################################################################################################
	
def fn_selection_amount(sel_value,r_ness,adjustment):
    # 
    value = (math.sqrt(sel_value)*r_ness)*adjustment
    return value


######################################################################################################################################
	
def proc_messagebox(message):
    old_handler = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2)
    pdb.gimp_message(str(message))
    pdb.gimp_message_set_handler(old_handler)
	
######################################################################################################################################
		
def proc_trim_away(img, layer, amount):
    #### Trim off pixels from a layer
    # set our feather settings
    our_feather = True
    our_feather_radius = 2
    #get feather settings
    feather = pdb.gimp_context_get_feather()
    feather_x, feather_y = pdb.gimp_context_get_feather_radius()
    # make sure that feather is set
    if not feather: pdb.gimp_context_set_feather(our_feather)
    # use our feather radius
    pdb.gimp_context_set_feather_radius(our_feather_radius, our_feather_radius)
    # select the contents of the layer
    pdb.gimp_image_select_item(img, 2, layer)
    # shrink the selection
    pdb.gimp_selection_shrink(img, amount)
    # invert selection
    pdb.gimp_selection_invert(img)
    # trim away pixels
    successful = pdb.gimp_edit_cut(layer)
    # deselect
    pdb.gimp_selection_none(img)
    # reset feather settings
    pdb.gimp_context_set_feather(feather)
    pdb.gimp_context_set_feather_radius(feather_x, feather_y)
	
    #### End of Proc_trim_away
	
######################################################################################################################################
	
def fn_get_mode(new_mode):
        # Use Legacy Modes if new Default Modes are set to Normal Mode
        temp_mode = 0
        if new_mode <> 0:
            if new_mode == 1:
                temp_mode = 1
            elif new_mode == 2:
                temp_mode = 57
            elif new_mode == 3:
                temp_mode = 59
            elif new_mode == 4:
                temp_mode = 60
            elif new_mode == 5:
                temp_mode = 36
            elif new_mode == 6:
                temp_mode = 55
            elif new_mode == 7:
                temp_mode = 31
            elif new_mode == 8:
                temp_mode = 42
            elif new_mode == 9:
                temp_mode = 33
            elif new_mode == 10:
                temp_mode = 35
            elif new_mode == 11:
                temp_mode = 54
            elif new_mode == 12:
                temp_mode = 30
            elif new_mode == 13:
                temp_mode = 43
            elif new_mode == 14:
                temp_mode = 53
            elif new_mode == 15:
                temp_mode = 23
            elif new_mode == 16:
                temp_mode = 45
            elif new_mode == 17:
                temp_mode = 44
            elif new_mode == 18:
                temp_mode = 48
            elif new_mode == 19:
                temp_mode = 49
            elif new_mode == 20:
                temp_mode = 50
            elif new_mode == 21:
                temp_mode = 51
            elif new_mode == 22:
                temp_mode = 32
            elif new_mode == 23:
                temp_mode = 52
            elif new_mode == 24:
                temp_mode = 34
            elif new_mode == 25:
                temp_mode = 46
            elif new_mode == 26:
                temp_mode = 47
            elif new_mode == 27:
                temp_mode = 41
            elif new_mode == 28:
                temp_mode = 37
            elif new_mode == 29:
                temp_mode = 38
            elif new_mode == 30:
                temp_mode = 39
            elif new_mode == 31:
                temp_mode = 40
            elif new_mode == 32:
                temp_mode = 24
            elif new_mode == 33:
                temp_mode = 25
            elif new_mode == 34:
                temp_mode = 26
            elif new_mode == 35:
                temp_mode = 27
            elif new_mode == 36:
                temp_mode = 56
        mode = temp_mode
			

        return mode
	


######################################################################################################################################
	
register(
    "python_fu_Custom_Font_Selection_Highlight_v01",
    "Provides options for adding highlights with selections",
    "Provides options for adding highlights with selections",
    "Skinnyhouse",
    "Open source (BSD 3-clause license)",
    "2024",
    "<Image>/Filters/Custom Fonts/Decorating Tools/Custom Font Selection Highlight v 0.1...",
    "RGB*",
    [
    #(PF_TOGGLE, "option", "Empty option to remove:", 0),
    (PF_SPINNER, "size_amount", "Highlight size \n(as percentage of character): ", 30, (1, 90, 1)),
    (PF_STRING, "thinner_string", "Characters to add\na different size of highlight: ", ""),
    (PF_SPINNER, "alt_size_amount", "Alternative highlight size \n(as percentage of character): ", 20.00, (0.2, 90.00, 0.2)),
    (PF_SPINNER, "step_amount", "Selection step: ", 1, (1, 10, 1)),
    (PF_SPINNER, "blur_amount", "Blur highlight amount: ", 2, (0, 90, 1)),
    (PF_OPTION, "color_option", "Highlight colour to use: ", 0, ("Use foreground colour", "Use background colour")),
    (PF_OPTION, "mode_option2", "New Mode \nto use with highlight: ", 0, ("Normal", "Dissolve","Color erase", "Merge", "Split", "Lighten only", "Luma/Luminance lighten only", "Screen", "Dodge", "Addition", "Darken only", "Luma/Luminance darken only", "Multiply", "Burn", "Linear burn", "Overlay", "Soft light", "Hard light", "Vivid light", "Pin light", "Linear light", "Hard mix", "Difference", "Exclusion", "Subtract", "Grain extract", "Grain merge", "Divide", "Hue (HSV)", "Saturation (HSV)", "Color (HSL)", "Value (HSL)", "Hue (LCH)", "Chroma (LCH)", "Color (LCH)", "Lightness (LCH)", "Luminance")),
    (PF_SPINNER, "pf_opacity", "Opacity level for highlight: ", 100,(5,100,1)),
    (PF_SPINNER, "x_off", "x offset for highlight: ", 0, (-50, 50, 1)),
    (PF_SPINNER, "y_off", "y offset for highlight: ", 0, (-50, 50, 1)),
    #(PF_SLIDER, "pf_azimuth", "Shine Azimuth:", 315, (0, 359, 1)),
    #(PF_TOGGLE, "pf_usecustomfont", "Combine the current character \nwith one from the custom font below:", 0),
    #(PF_FILENAME, "pf_names_xcffilename", ".xcf custom font file:", 0),
    #(PF_INT, "number", "Number?", 50),
    #(PF_FLOAT, "angle", "Angle", 3.14159),
    #(PF_STRING, "word", "Word", "Zebrafish!"),
    #(PF_TEXT, "text", "Some Text",
    #(PF_RADIO, "imagefmt", "Image format", "jpg", (("png", "png"), ("jpg", "jpg"))),
    #(PF_OPTION, "option", "Option", 2, ("Mouse", "Cat", "Dog", "Horse")),
    #(PF_SPINNER, "size", "Pixel Size", 50, (1, 8000, 1)),
    #(PF_FONT, "font", "Font", "Sans"),
    #(PF_BRUSH, "brush", "Brush", None),
    #(PF_PATTERN, "pattern", "Pattern", None),
    #(PF_GRADIENT, "gradient", "Gradient", None),
    #(PF_PALETTE, "palette",  "Palette", ""),
    #(PF_VECTORS, "vectors", "Vectors", None),
    ],
    [],
    do_stuff)

main()
